@@ -1,38 +1,18 @@
-########################################################################
-# FindBin::libs
-#
-# use $FindBin::Bin to search for 'lib' directories and use them.
-#
-# default action is to look for dir's named "lib" and silently use
-# the lib's without exporting anything. print turns on a short 
-# message with the abs_path results, export pushes out a variable
-# (default name is the base value), verbose turns on decision output
-# and print. export takes an optional argument with the name of a
-# variable to export.
-#
-# Copyright (C) 2003-2012, Steven Lembark, Workhorse Computing.
-# This code is released under the same terms as Perl-5.8
-# or any later version of Perl.
-# 
-########################################################################
-
 ########################################################################
 # housekeeping
 ########################################################################
 
-package         # hak to avoid indexing the extra module
-FindBin::libs;
+package FindBin::libs   v2.12.01;
 
-use v5.8;
+use v5.14;
 use strict;
 
 use FindBin;
-use Symbol;
 
 use File::Basename;
 
-use Carp    qw( croak           );
-use Symbol  qw( qualify_to_ref  );
+use Carp    qw( croak                   );
+use Symbol  qw( qualify qualify_to_ref  );
 
 use File::Spec::Functions
 qw
@@ -57,56 +37,59 @@ BEGIN
     # cannot handle the rooted system being linked
     # back to itself.
 
-    use Cwd qw( &cwd );
-
-    my $abs = Cwd->can( 'abs_path'  )
-    or die "Odd: Cwd cannot 'abs_path'\n";
+    use Cwd qw( &abs_path &cwd );
 
     if
     (
-        eval { $abs->( '//' );  $abs->( cwd ); 1 }
+        # abs_path has a fixed bug dealing with infinite
+        # recursion. if upping the version of Cwd does 
+        # not fix this then the only other test I can 
+        # think of is ( -e '/.' && -e '/..' && -e '/../.' )
+
+        eval 
+        {
+            abs_path '//';
+            abs_path cwd
+        }
     )
     {
-        # nothing more to do: abs_path works.
-    }
-    elsif
-    (
-        $abs = File::Spec::Functions->can( 'rel2abs'  )
-    )
-    {
-        # ok, we have a substitute
+        # abs_path seems clean on this platform.
     }
     else
     {
-        die "Cwd fails abs_path test && lacks 'rel2abs'\n";
-    }
+        # abs_path seems to be having problems,
+        # fix is to stub it out.
+        #
+        # undef avoids nastygram.
 
-    my $ref = qualify_to_ref 'abs_path', __PACKAGE__;
+        my $ref = qualify_to_ref 'abs_path', __PACKAGE__;
 
-    undef &{ *$ref };
+        my $sub = File::Spec::Functions->can( 'rel2abs' );
 
-    *{ $ref } = $abs;
+        undef &{ $ref };
+
+        *$ref = $sub
+    };
 }
 
 ########################################################################
 # package variables 
 ########################################################################
 
-our $VERSION = '1.062_01';
-$VERSION = eval $VERSION;
-
 my %defaultz = 
 (
     base    => 'lib',
     use     => undef,
+    blib    => undef,   # prefer ./blib at the first level
 
     subdir  => '',      # add this subdir also if found.
     subonly => undef,   # leave out lib's, use only subdir.
     export  => undef,   # push variable into caller's space.
+    append  => undef,   # push onto existing array (vs. overwrite)
     verbose => undef,   # boolean: print inputs, results.
     debug   => undef,   # boolean: set internal breakpoints.
 
-    print   => undef,   # display the results
+    print   => 1,       # display the results
 
     p5lib   => undef,   # prefix PERL5LIB with the results
 
@@ -154,10 +137,10 @@ my $find_libs
     # after that splitpath can grab the directory 
     # portion for future use.
 
-    my ( $Bin ) = $argz{ Bin } =~ m{^ (.+) }xs;
+    my ( $Bin ) = ( $argz{ Bin } =~ m{^ (.+) }xs );
 
     print STDERR "\nSearching $Bin for '$base'...\n"
-        if $verbose;
+    if $verbose;
 
     my( $vol, $dir ) = splitpath $Bin, 1;
 
@@ -171,6 +154,7 @@ my $find_libs
 
     my @libz    = ();
 
+    PATH:
     for( 1 .. @dirpath )
     {
         # note that catpath is extraneous on *NIX; the 
@@ -183,7 +167,11 @@ my $find_libs
         # to eval the check for subdir's. 
 
         my $abs
-        = eval { abs_path catpath $vol, ( catdir @dirpath, $base ), $empty }
+        = eval
+        {
+            abs_path
+            catpath $vol, ( catdir @dirpath, $base ), $empty
+        }
         || '';
 
         my $sub
@@ -198,9 +186,11 @@ my $find_libs
         {
             if( $dir && -d $dir && ! exists $found{ $dir } )
             {
-                $found{ $dir } = 1;
+                $found{ $dir } = ();
 
                 push @libz, $dir;
+
+                last if $argz{ scalar };
             }
         }
 
@@ -211,11 +201,14 @@ my $find_libs
     # (including volume) walking up the path 
     # from $FindBin::Bin -> root.
     #
+    # no libs found is empty list or undef for 
+    # scalar.
+    #
     # passing it back as a list isn't all that
     # painful for a few paths.
 
     wantarray ? @libz : \@libz
-};;
+};
 
 # break out the messy part into a separate block.
 
@@ -232,17 +225,31 @@ my $handle_args
     %argz
     = map
     {
+        my $use_undef
+        = do
+        {
+            my %a   = ();
+            @a{ qw( export ignore ) } = ();
+            \%a
+        };
+
         my ( $k, $v ) = split '=', $_, 2;
 
-        $k =~ s{^ (?:!|no) }{}x
-        ? ( $k => undef )
-        : ( $k => ( $v || '' )  )
+        exists $use_undef->{ $k }
+        or $v //= 1;
+
+        # "no" inverts the sense of the test.
+
+        $k =~ s{^no}{}
+        and $v  = ! $v;
+
+        ( $k => $v )
     }
     @_;
 
     # stuff "debug=1" into your arguments and perl -d will stop here.
 
-    $DB::single = 1 if defined $argz{debug};
+    $DB::single = 1 if defined $argz{ debug };
 
     # default if nothing is supplied is to use the result;
     # otherwise, without use supplied either of export or
@@ -270,27 +277,34 @@ my $handle_args
     # now apply the defaults, then sanity check the result.
     # base is a special case since it always has to exist.
     #
-    # if $argz{export} is defined but false then it takes
-    # its default from $argz{base}.
+    # if $argz{ export } is defined but false then it takes
+    # its default from $argz{ base }.
 
-    exists $argz{$_} or $argz{$_} = $defaultz{$_}
-    for keys %defaultz;
+    while( my($k,$v) = each %defaultz )
+    {
+        # //= doesn't work here since undef may be a 
+        # legit default.
 
-    exists $argz{base} && $argz{base} 
+        exists $argz{ $k }
+        or
+        $argz{ $k } = $v;
+    }
+
+    exists $argz{ base } && $argz{ base } 
     or croak "Bogus FindBin::libs: missing/false base argument, should be 'base=NAME'";
 
-    defined $argz{export} and $argz{export} ||= $argz{base};
+    exists $argz{ export }
+    and
+    $argz{ export } //= $argz{ base };
 
     $argz{ ignore } =
     [
-        grep { $_ }
-        split /\s*,\s*/,
-        $argz{ignore}
+        grep { $_ } split /\s*,\s*/, $argz{ ignore }
     ];
 
-    $verbose = defined $argz{verbose};
+    $verbose = defined $argz{ verbose };
 
-    my $base = $argz{base};
+    my $base = $argz{ base };
 
     # now locate the libraries.
     #
@@ -303,13 +317,13 @@ my $handle_args
 
     for( @{ $argz{ ignore } } )
     {
-      if( my $dir = eval { abs_path catdir $_, $base } )
-      {
-        if( -d $dir )
+        if( my $dir = eval { abs_path catdir $_, $base } )
         {
-          $found{ $dir } = 1;
+            if( -d $dir )
+            {
+                $found{ $dir } = 1;
+            }
         }
-      }
     }
 };
 
@@ -324,57 +338,92 @@ sub import
     # of inside find_libs allows people to use saner
     # untainting plans via find_libs.
 
-    @libz   = map { m{ (.+) }x } @libz;
+    @libz   = map { m{ (.+) }xs } @libz;
 
     my $caller = caller;
 
-    if( $verbose || defined $argz{print} )
+    if( $verbose || defined $argz{ print } )
     {
         local $\ = "\n";
         local $, = "\n\t";
 
         print STDERR "Found */$argz{ base }:", @libz
-    }
-
-    if( $argz{export} )
-    {
-        print STDERR join '', "\nExporting: @", $caller, '::', $argz{export}, "\n"
-        if $verbose;
-
-        # Symbol this is cleaner than "no strict" 
-        # for installing the array.
-
-        my $ref = qualify_to_ref $argz{ export }, $caller;
-
-        *$ref = \@libz;
-    }
-
-    if( defined $argz{ p5lib } )
-    {
-        # stuff the lib's found at the front of $ENV{ PERL5LIB }
-
-        ( substr $ENV{ PERL5LIB }, 0, 0 ) = join ':', @libz, ''
-        if @libz;
-
-        print STDERR "\nUpdated PERL5LIB:\t$ENV{ PERL5LIB }\n"
         if $verbose;
     }
 
-    if( $argz{use} && @libz )
+    if( $argz{ export } )
     {
-        # this obviously won't work if lib ever depends 
-        # on the caller's package.
-        #
-        # it does avoids issues with -T blowing up on the
-        # old eval technique.
+        # this has to run in order to install variables that
+        # the caller is expecting to exist at runtime -- even
+        # if they are empty/undef at the end of it.
 
-        require lib;
+        my $ref     = qualify_to_ref $argz{ export }, $caller;
 
-        lib->import( @libz );
+        if( $verbose )
+        {
+            my $dest    = qualify $argz{ export }, $caller;
+
+            $argz{ scalar }
+            ? print STDERR "\nExporting: \$$dest\n"
+            : print STDERR "\nExporting: \@$dest\n"
+            ;
+        }
+
+        if( $argz{ scalar } )
+        {
+            *$ref 
+            = @libz
+            ? \$libz[0]
+            : \( my $a = '' )
+            ;
+        }
+        elsif
+        (
+            $argz{ append } 
+            and
+            my $ary = *{ $ref }{ ARRAY }
+        )
+        {
+            push @$ary, @libz;
+        }
+        else
+        {
+            *$ref = \@libz
+        }
+    }
+
+    # no 'else', these are not exclusive
+
+    if( @libz )
+    {
+        if( defined $argz{ p5lib } )
+        {
+            # stuff the lib's found at the front of $ENV{ PERL5LIB }
+            # yes, virginia, substr is an lvalue -- and saner than
+            # dealing with \Q and a regex on arbitrary paths.
+
+            ( substr $ENV{ PERL5LIB }, 0, 0 ) = join ':', @libz, '';
+
+            print STDERR "\nUpdated PERL5LIB:\t$ENV{ PERL5LIB }\n"
+            if $verbose;
+        }
+
+        if( $argz{ use } )
+        {
+            # this obviously won't work if lib ever depends 
+            # on the caller's package.
+            #
+            # it does avoids issues with -T blowing up on the
+            # old eval technique.
+
+            require lib;
+
+            lib->import( @libz );
+        }
     }
 
     0
-};
+}
 
 # keep require happy
 
@@ -384,16 +433,11 @@ __END__
 
 =head1 NAME
 
-FindBin::libs_5_8 - stable version of code for older
-perl installations.
+FindBin::libs - locate and a 'use lib' or export 
+directories based on $FindBin::Bin.
 
 =head1 SYNOPSIS
 
-FindBin::libs locates and performs a 'use lib' for 
-directories along the path of $FindBin::Bin to automate 
-locating modules.  Uses File::Spec and Cwd's abs_path to 
-accomodate multiple O/S and redundant symlinks.
-
     # search up $FindBin::Bin looking for ./lib directories
     # and "use lib" them.
 
@@ -517,11 +561,24 @@ that
 
 will search for directories named "altlib" and "use lib" them.
 
-=head3 Exporting a variable: 'export'
+=head3 Exporting a variable: "export", "scalar", "append"
 
-The 'export' option will push an array of the directories found
-and takes an optional argument of the array name, which defaults 
-to the basename searched for:
+=over 4
+
+=item "export"
+
+This installs the results of locating directories into the caller's 
+space. Without any argument, export pushes out a variable named after 
+the located [sub]dir; an argument can be supplied to give the variable 
+name. Without the "scalar" option, the exported variable will be an 
+array in increasing order of "distance" (i.e., "up" the file tree); 
+with the "scalar" option only the first (i.e., "nearest") path is 
+exported.
+
+If "export" is given then "nouse" is assumed; using both leaves the 
+variable exported and its contents handed to "use lib".
+
+For example:
 
     use FindBin::libs qw( export );
 
@@ -551,7 +608,7 @@ exports @meta while
 
     use FindBin::libs qw( export=metadirs base=meta );
 
-exports @metadirs.
+exports @metadirs as a list of paths ending in "/meta".
 
 The use and export switches are not exclusive:
 
@@ -560,21 +617,85 @@ The use and export switches are not exclusive:
 will locate "lib" directories, use lib them, and export 
 @mylibs into the caller's package. 
 
+=item "scalar"
+
+Only searches for the first directory, which is exported (or
+overwritten) as a scalar rather than array. For example, if
+a project directory has ./bin and ./etc dir's then #! code in
+bin with
+
+    use FindBin::libs qw( export scalar base=etc );
+
+will have an $etc variable with the absolute path to ./bin/../etc.
+For configuration varibles this is usually what you want and allows
+for "$etc/Foo.conf" rather than "$etc[0]/Foo.conf".
+
+=item "append"
+
+Sometimes it's simpler to accumulate multiple searches into a 
+single array. Say for ./etc dir's in collection of standard
+locations.
+
+In that case:
+
+    use FindBin::libs qw( export=etc base=foo subdir=etc );
+    use FindBin::libs qw( export=etc base=bar subdir=etc append );
+
+produces something like
+
+    (
+        /path/to/foo/etc
+        /path/to/bar/etc
+    )
+
+without append @etc will have only ./bar/etc since the array would
+be overwritten with each call to import.
+
+=back
+
 =head3 Subdirectories
 
 The "subdir" and "subonly" settings will add or 
 exclusively use subdir's. This is useful if some
 of your lib's are in ../lib/perl5 along with 
-../lib (subdir=perl5) or all of the lib's are 
-in ../lib/perl5 (subonly=perl5).
+../lib or all of the lib's are in ../lib/perl5.
+
+These could be handled with:
+
+    use FindBin::libs;
+    use FindBin::libs qw( subdir=perl5 subonly );
+
+which uses the "lib" dir's along with any lib/perl5 dirs.
 
 This can also be handy for locating subdir's used
 for configuring packages:
 
     use FindBin::libs qw( export base=config subonly=mypackage );
 
-Will leave @config with any "mypackage" holding
-any "mypackage" subdir's.
+Will leave @config containing any mypackage dir's found up
+the tree, nearest to closest.
+
+The array format is convienent for locating configuration files
+shared between projects in separate, sibling directories. For
+example given:
+
+    ./proj/Foo/etc
+    ./proj/etc
+
+with
+
+    use FindBin::libs qw( export subdir=etc subonly )
+
+will export @etc with qw( ../proj/Foo/etc ../proj/etc ) in lexical
+order by distance from the #! code. At that point
+
+    use List::Util qw( first );
+
+    my $path = first { -e "$_/Global.config" } @etc;
+
+will locate the nearest "Global.confg" file. Note that this is 
+not the same as using "scalar" since that will export 
+$etc with only ./Foo/etc. 
 
 =head3 Setting PERL5LIB: p5lib
 
@@ -605,8 +726,6 @@ to something like:
 This can make controlling the paths used simpler and avoid
 the use of symlinks for some testing (see examples below).
 
-Note that "p5lib" and "nouse" are proably worth 
-
 =head2 Skipping directories
 
 By default, lib directories under / and /usr are
@@ -654,7 +773,7 @@ having to modify a single line of code.
 Say your sandbox is in ./sandbox and you are currently
 working in ./sandbox/projects/package/bin on a perl
 executable. You may have some number of modules that
-are specific -- or customized -- for this package, 
+are specific -- or customized -- for this pacakge, 
 share some modules within the project, and may want 
 to use company-wide modules that are managed out of 
 ./sandbox in development. All of this lives under a 
@@ -794,7 +913,7 @@ allow:
 
 to check the code.
 
-=back 
+=back
 
 =head1 Notes
 
@@ -916,10 +1035,10 @@ where '-e' would have succeded.
 This is used for portability in dis- and re-assembling 
 directory paths based on $FindBin::Bin.
 
-=item libs_curr.pm
+=item Older code.
 
-This is installed if $^V indicates that the running
-perl is >= v5.10.
+FindBin::libs_5_8.pm is installed if $^V indicates
+that the running perl is prior to v5.10.
 
 =back
 
@@ -961,21 +1080,39 @@ rather than checking this every time at startup.
 
 For the moment, at least, this seems to work.
 
-=back 
+=back
 
 =head1 SEE ALSO
 
-=head2 Module::FromPerlVer
+=over 4
+
+=item Module::FromPerlVer
 
 Explains where the installed version comes from.
 
+=item File::Copy::Recursive
+
+Note: Due to issues with File::Copy::Recusive, this is
+using File::Copy::Recursive::Reduced and a version directory
+for the moment. Once the Recursive module has been dealt with
+Module::FromPerlVer shoud Just Work.
+
+=back
+
+=head2 Cwd
+
+A bug in Cwd was fixed with 3.73. That should, hopefully,
+fix the issue with FB::l croaking perl during testing.
+If it does not then I will have to modify the sanity check
+for using abs_path vs. rel2abs.
+
 =head1 AUTHOR
 
 Steven Lembark, Workhorse Computing <lembark@wrkhors.com>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2012, Steven Lembark, Workhorse Computing.
+Copyright (C) 2003-2014, Steven Lembark, Workhorse Computing.
 This code is released under the same terms as Perl-5.24
 or any later version of Perl.
 
